/*
 * cli_conf.c   DHCPv6 Client Configuration
 *
 * Version:     @(#)cli_conf.c   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#include "compat.h"
#include <db.h>
#include "localdefs.h"
#include "u_timer.h"
#include "u_buf.h"
#include "dispatch.h"
#include "dh6.h"
#include "ext6.h"
#include "dh6_adm.h"
#include "dh6_db.h"
#include "parse.h"
#include "dh6_if.h"
#include "dh6_cli.h"

extern int	SolicitPbit;
extern int	SolicitCbit;
extern int	RequestLife;
extern int	RequestQbit;

int read_dh6_client_conf( void )
{
	char fname[60];
	char buf[100];
	int type;
	FILE *f;

	sprintf(fname, "%s%s", CLI_CONF_PATH, DH6CLI_CONF);
	if ((f=fopen(fname, "r+")) == NULL)
		log_sys(1401, "fopen");

	rewind(f);
	parse_init();
	do {
		type = get_next_token(f, buf);
		switch (type) {
		case CLI_SOLICIT_PBIT:
			type = get_next_token(f, buf);
			if (type & DIGIT) {
				SolicitPbit = atoi(buf);
				continue;
			}
			log_warn(1402, "incorrect SolicitPbit format");
			break;
		case CLI_SOLICIT_CBIT:
			type = get_next_token(f, buf);
			if (type & DIGIT) {
				SolicitCbit = atoi(buf);
				continue;
			}
			log_warn(1403, "incorrect SolicitCbit format");
			break;
		case CLI_REQUEST_LIFE:
			type = get_next_token(f, buf);
			if (type & DIGIT) {
				RequestLife = atoi(buf);
				continue;
			}
			log_warn(1404, "incorrect RequestLife format");
			break;
		case CLI_REQUEST_QBIT:
			type = get_next_token(f, buf);
			if (type & DIGIT) {
				RequestQbit = atoi(buf);
				continue;
			}
			log_warn(1405, "incorrect RequestQbit format");
			break;
		}
	} while (type != EOF);

	fclose(f);
	return 0;
}

int seek_match( char *id, int type )
{
	if ( !isascii(id[0]) )
		return type;

	switch ( tolower(id[0]) ) {
		case 's':
			if (!strcasecmp (id, "SolicitPbit"))
				return CLI_SOLICIT_PBIT;
			if (!strcasecmp (id, "SolicitPbit"))
				return CLI_SOLICIT_PBIT;
		case 'r':
			if (!strcasecmp (id, "RequestLife"))
				return CLI_REQUEST_LIFE;
			if (!strcasecmp (id, "RequestQbit"))
				return CLI_REQUEST_QBIT;
	}
	return type;
}
