/*
 * dh6_if.h    	DHCPv6 Interface Structures
 *
 * Version:     @(#)dh6_if.h   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#ifndef __ND6_IF_H
#define __ND6_IF_H

/*
 * per-interface information
 *
 * XXX may need to handle multiple link local addresses per interface
 */

struct if_info {
	struct if_info		*next;			/* next interface */
	struct if_info		*prev;			/* previous interface */

	char			 ifi_name[IFNAMSIZ];	/* interface name */
	struct in6_addr		 ifi_lladdr;		/* link-local address */
	int                      ifi_ifindex;           /* interface index */
	UNION_LIST 		 ifi_dhcpv6;       	/* DHCPv6 addresses */
	pid_t			 ifi_pid;		/* DHCPv6 procedd ID */
	u_int16_t		 ifi_dh6:1,		/* if DHCPv6 is running */
				 rsvd:15;
};

typedef struct if_info	IF_INFO;	/* interface structure */
#define ifi_ifname(ifi)  ((ifi)->ifi_name)
#define ifi_ifindex(ifi) ((ifi)->ifi_ifindex)
#define ifi_lladdr(ifi)  &((ifi)->ifi_lladdr)

extern void init_ifi_table( void );
extern int get_ifi_table( void );
extern struct if_info * seek_ifi_byname(char *, int);
extern struct if_info * seek_ifi_byindex(int);
extern struct if_info * seek_ifi_bypid(pid_t);
extern int if_info_kernel_set ( int, char *, int, int );
extern int if_info_kernel_get ( int, char *, int, int * );
extern void print_ifi_table( void );
extern void ifi_kill_dhcpv6( void );

#endif /* __ND6_IF_H */
