/*
 * nd6_if.c
 *
 * errors 600 - 699
 */

#include "dh6all.h"
#include "localdefs.h"
#include "u_timer.h"
#include "dispatch.h"
#include "u_buf.h"
#include "dh6.h"
#include "ext6.h"
#include "nd6_if.h"
#include "dh6_cli.h"
#include "dh6_adm.h"

UNION_LIST 	ifi_table;

void init_ifi_table( void )
{
	ifi_table.ifi.head = NULL;
	ifi_table.ifi.tail = NULL;
}

int get_ifi_table( void )
{
	u_char buf[8192];
	u_char *buf_end;
	struct ifconf ifc;
	struct ifreq *ifr;
	struct if_info *ifi;
	struct sockaddr_in6 *sin6;
	int s;

	if ((s=socket(AF_INET6, SOCK_DGRAM, 0)) < 0)
		log_sys(201, "socket");

	ifc.ifc_len = sizeof(buf);
	ifc.ifc_buf = (char *)buf;

	if (ioctl(s, SIOCGIFCONF, (char *)&ifc) < 0)
		log_fatal(202,"SIOCGIFCONF failed");
	buf_end = buf + ifc.ifc_len;

	ifr = ifc.ifc_req;
	while ((u_char *)ifr < buf_end) {
		sin6 = (struct sockaddr_in6 *)&ifr->ifr_addr;
#ifdef DH6_DEBUG
        	if ( look_xmit_more() ) {
			if ( ifr->ifr_addr.sa_family == AF_LINK ) {
                		struct sockaddr_dl *sdl = 
					(struct sockaddr_dl *)sin6;
				char ss[20];
				strncpy(ss, sdl->sdl_data, sdl->sdl_nlen);
				log_info("AF_LINK: %s index %d", 
						ss, sdl->sdl_index);
			}
			else if ( ifr->ifr_addr.sa_family == AF_INET6 )
				log_info("AF_INET6: %s",
				inet_ntop(AF_INET6,&sin6->sin6_addr,16,NULL));
			else if ( ifr->ifr_addr.sa_family == AF_INET )
				log_info("AF_INET: %s", 
				inet_ntoa(((struct sockaddr_in*)sin6)->sin_addr));
		}
#endif
		if ( ifr->ifr_addr.sa_family == AF_INET6 &&
		     IN6_IS_ADDR_LINKLOCAL(&sin6->sin6_addr) ) {
			if ( (ifi = u_alloc_node()) == NULL )
				return -1;
			memset(ifi, 0, sizeof(struct if_info));
			strcpy(ifi->ifi_name, ifr->ifr_name);
			memcpy(&ifi->ifi_lladdr, &sin6->sin6_addr,
				sizeof(struct in6_addr));
			u_queue_tail(&ifi_table, ifi);
		}
		ifr = (struct ifreq *)((char *)ifr + sizeof(ifr->ifr_name) +
			(sizeof(ifr->ifr_addr) > ifr->ifr_addr.sa_len ?
			 sizeof(ifr->ifr_addr) : ifr->ifr_addr.sa_len ));
	}

	ifr = ifc.ifc_req;
	while ((u_char *)ifr < buf_end) {
               	struct sockaddr_dl *sdl;
		if ( ifr->ifr_addr.sa_family != AF_LINK )
			goto get_ifi_table_skip;
		sdl = (struct sockaddr_dl *)&ifr->ifr_addr;
		for (ifi = ifi_table.ifi_head; 
		     ifi != NULL;
		     ifi = ifi->next) {
			if ( !strncmp(ifi->ifi_name,
				      sdl->sdl_data,
				      sdl->sdl_nlen) ) {
				ifi->ifi_ifindex = sdl->sdl_index;
			}
		}
get_ifi_table_skip:
		ifr = (struct ifreq *)((char *)ifr + sizeof(ifr->ifr_name) +
			(sizeof(ifr->ifr_addr) > ifr->ifr_addr.sa_len ?
			 sizeof(ifr->ifr_addr) : ifr->ifr_addr.sa_len ));
	}
	close(s);
	return 0;
}

struct if_info * seek_ifi_byname( char *name, int len )
{
	struct if_info *ifi;

	for (ifi = ifi_table.ifi_head;
	     ifi != NULL;
	     ifi = ifi->next) {
		if ( !strncmp(ifi->ifi_name,name,len) )
			return ifi;
	}
	return NULL;
}

struct if_info * seek_ifi_byindex( int ifindex )
{
	struct if_info *ifi;

	for (ifi = ifi_table.ifi_head;
	     ifi != NULL;
	     ifi = ifi->next) {
		if ( ifi->ifi_ifindex = ifindex )
			return ifi;
	}
	return NULL;
}

struct if_info * seek_ifi_bypid( pid_t pid )
{
	struct if_info *ifi;

	for (ifi = ifi_table.ifi_head;
	     ifi != NULL;
	     ifi = ifi->next) {
		if ( ifi->ifi_pid == pid )
			return ifi;
	}
	return NULL;
}

void ifi_kill_dhcpv6( void )
{
	struct if_info *ifi;

	for (ifi = ifi_table.ifi_head;
	     ifi != NULL;
	     ifi = ifi->next) {
		if ( ifi->ifi_dh6 )
			kill(ifi->ifi_pid, SIGKILL);
	}
}

void print_ifi_table( void )
{
	struct if_info *ifi;
	int n = 1;

	for (ifi = ifi_table.ifi_head;
	     ifi != NULL;
	     ifi = ifi->next) {
		log_info("if%d: name = %s, index = %d", 
				n++, ifi->ifi_name, ifi->ifi_ifindex);
		log_info("     lladdr = %s", 
				inet_ntop(AF_INET6,&(ifi->ifi_lladdr),16,NULL));
		log_info("     pid = %d,  dhcpv6: %d", 
				ifi->ifi_pid, ifi->ifi_dh6);
	}
}
