/*
 * svr_assign.c DHCPv6 Server Address Assignment
 *
 * Version:     @(#)svr_assign.c   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#include "compat.h"
#include <db.h>
#include "localdefs.h"
#include "u_timer.h"
#include "u_buf.h"
#include "dispatch.h"
#include "dh6.h"
#include "ext6.h"
#include "dh6_adm.h"
#include "dh6_db.h"
#include "parse.h"
#include "dh6_if.h"
#include "dh6_svr.h"

extern int debug;

#ifdef DH6_DB

int dh6_any_idle_dh6addr( void )
{
	FILE *f;
	char fname[60];

	if ( dh6db_any_idle_dh6addr() )
		return 1;
	sprintf(fname, "%s%s", SVR_CONF_PATH, DH6_NEWADDR);
	if ((f=fopen(fname, "r+")) == NULL)
		log_sys(3401, "fopen");
#ifdef DH6_DEBUG
        if ( look_db() )
		log_info("dh6_any_idle_dh6addr(): open %s", fname);
#endif
	if (seek_identifer(SVR_ADDR_FROM, f) != EOF) {
		fclose(f);
		return 1;
	}
	fclose(f);
	return 0;
}

static inline void incr_net_int32(u_int32_t *a)
{
	*a = htonl( ntohl(*a)+1 );
}

int inet_get_next_addr(struct in6_addr *next_addr)
{
	if (next_addr->s6_addr32[3] != ~0)
		incr_net_int32(next_addr->s6_addr32+3);
	else if (next_addr->s6_addr32[2] != ~0) {
		incr_net_int32(next_addr->s6_addr32+2);
		next_addr->s6_addr32[3] = 0;
	} else if (next_addr->s6_addr32[1] != ~0) {
		incr_net_int32(next_addr->s6_addr32+1);
		next_addr->s6_addr32[2] = 0;
		next_addr->s6_addr32[3] = 0;
	} else if (next_addr->s6_addr32[0] != ~0) {
		incr_net_int32(next_addr->s6_addr32);
		next_addr->s6_addr32[1] = 0;
		next_addr->s6_addr32[2] = 0;
		next_addr->s6_addr32[3] = 0;
	} else
		memset(next_addr, 0, IPV6_ADDR_LEN);
#ifdef DH6_DEBUG
        if ( look_db_more() )
		log_info("inet_get_next_addr(): %s",
			inet_ntop(AF_INET6, next_addr, IPV6_ADDR_LEN, NULL));
#endif
	return 0;
}

inline int inet_cmp(struct in6_addr *addr1, struct in6_addr *addr2)
{
	return memcmp(addr1, addr2, IPV6_ADDR_LEN);
}

int dh6_get_idle_dh6addr( struct svr_client *sc )
{
	FILE *f, *f2;
	char fname[60], fname2[64];
	char from[100], to[100], buf[100];
	struct in6_addr in6from, in6to;
	int type, result;

	if ( dh6db_get_idle_dh6addr(sc) )
		return 1;
	sprintf(fname, "%s%s", SVR_CONF_PATH, DH6_NEWADDR);
	sprintf(fname2, "%s.old", fname);
	rename(fname, fname2);
	if ((f=fopen(fname, "w+")) == NULL)
		log_sys(3402, "fopen %s", fname);
	if ((f2=fopen(fname2, "r+")) == NULL)
		log_sys(3403, "fopen %s", fname2);
#ifdef DH6_DEBUG
        if ( look_db() ) {
		log_info("dh6_get_idle_dh6addr(): open %s", fname);
		log_info("open %s", fname2);
	}
#endif
	if (fcopy_until(SVR_ADDR_FROM, f, f2) == EOF) 
		goto _dh6_get_idle_dh6addr_exit;
	type = get_next_token(f2, from);
	if ( !(type & STRING) )
		goto _dh6_get_idle_dh6addr_exit;
	type = get_next_token(f2, buf);
	if (type != SVR_ADDR_TO)
		goto _dh6_get_idle_dh6addr_exit;
	type = get_next_token(f2, to);
	if ( !(type & STRING) )
		goto _dh6_get_idle_dh6addr_exit;
	type = get_next_token(f2, buf);
	if (type != SVR_ADDR_PREFIX)
		goto _dh6_get_idle_dh6addr_exit;
	type = get_next_token(f2, buf);
	if ((type & 0xff00) != DIGIT) 
		goto _dh6_get_idle_dh6addr_exit;

	inet_pton(AF_INET6, from, &in6from);
	inet_pton(AF_INET6, to, &in6to);
	sc->sc_prfsize = atoi(buf);
	memcpy(&sc->sc_cliaddr, &in6from, IPV6_ADDR_LEN);
	if ( (result=inet_cmp(&in6from, &in6to)) != 0) {
		inet_get_next_addr(&in6from);
		fprintf(f, "From    \"%s\"\n", 
			inet_ntop(AF_INET6, &in6from, IPV6_ADDR_LEN, NULL));
		fprintf(f, "To      \"%s\"\n", to);
		fprintf(f, "Prefix  %d\n", sc->sc_prfsize);
	}

	fcopy_until(EOF, f, f2);
	fclose(f2);
	fclose(f);
	unlink(fname2);
	return 1;
	
_dh6_get_idle_dh6addr_exit:
	fclose(f2);
	fclose(f);
	unlink(fname);
	rename(fname2, fname);
	return 0;
}

int dh6_get_requested_dh6addr( struct svr_client *sc )
{
	return dh6db_get_requested_dh6addr(sc);
}

int dh6_release_dh6addr( struct svr_client *sc )
{
	return dh6db_release_dh6addr(sc);
}

#else /* DH6_DB */

static struct addr_pool {
    	u_int8_t inuse;
    	u_int8_t prfsize;
    	char 	 addr[64];
} tmp_addr_pool[] = {
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5e0"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5e1"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5e2"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5e3"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5e4"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5e5"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5e6"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5e7"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5e8"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5e9"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5ea"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5eb"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5ec"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5ed"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5ee"},
	{0, 128, "5f02:3000:84b1:7600:0:900:2b39:b5ef"},
	{0,  0, ""}
    };

int dh6_any_idle_dh6addr( void )
{
	struct addr_pool *p;
	for (p = tmp_addr_pool; p->addr[0] != 0; p++) {
		if ( !p->inuse )
			return 1;
	}
	return 0;
}

int dh6_get_idle_dh6addr( struct svr_client *sc )
{
	struct addr_pool *p;
	for (p = tmp_addr_pool; p->addr[0] != 0; p++) {
		if ( !p->inuse ) {
			inet_pton(AF_INET6, p->addr, &sc->sc_cliaddr);
			sc->sc_prfsize = p->prfsize;
			p->inuse = 1;
			return 1;
		}
	}
	return 0;
}

int dh6_get_requested_dh6addr( struct svr_client *sc )
{
	static struct in6_addr addr;
	struct addr_pool *p;
	for (p = tmp_addr_pool; p->addr[0] != 0; p++) {
		if ( p->inuse ) 
			continue;
		inet_pton(AF_INET6, p->addr, &addr);
		if ( !memcmp(&sc->sc_cliaddr, &addr, IPV6_ADDR_LEN) ) {
			sc->sc_prfsize = p->prfsize;
			p->inuse = 1;
			return 1;
		}
	}
	return 0;
}

int dh6_release_dh6addr( struct svr_client *sc )
{
	static struct in6_addr addr;
	struct addr_pool *p;
	for (p = tmp_addr_pool; p->addr[0] != 0; p++) {
		if ( !p->inuse ) 
			continue;
		inet_pton(AF_INET6, p->addr, &addr);
		if ( !memcmp(&sc->sc_cliaddr, &addr, IPV6_ADDR_LEN) ) {
			p->inuse = 0;
			return 1;
		}
	}
	return 0;
}

#endif /* DH6_DB */
