/*
 * svr_db.c    	Database Routines for DHCPv6 Server
 *
 * Version:     @(#)svr_db.c   1.0     Dec-8-96
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed WITHOUT ANY WARRANTY and even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Many thanks to Jim Bound in Digital Equipment Corp.
 * Appreciate any further work on this release.
 *
 * Authors:     Yunzhou Li (yunzhou@iol.unh.edu)
 *
 */

#include "compat.h"
#include <db.h>
#include "localdefs.h"
#include "u_timer.h"
#include "u_buf.h"
#include "dispatch.h"
#include "dh6.h"
#include "ext6.h"
#include "dh6_adm.h"
#include "dh6_db.h"
#include "parse.h"
#include "dh6_if.h"
#include "dh6_svr.h"

/*
 *	Be careful!!! The length of a key should be multiple of 16!
 */
int dh6db_alloc_key(DBT *key, void *entry)
{
	struct svr_client *sc = entry;
	key->size = 2*IPV6_ADDR_LEN;
	key->data = malloc(key->size);
	dh6db_copy(key->data, &sc->sc_lladdr, IPV6_ADDR_LEN);
	dh6db_copy(((u_int8_t *)key->data)+IPV6_ADDR_LEN, 
		&sc->sc_rlyaddr, IPV6_ADDR_LEN);
}

int dh6db_alloc_rec(DBT *rec, void *entry)
{
	struct svr_client *sc = entry;
	struct svr_record *srec;
	struct timeval tv;

	rec->size = sizeof(struct svr_record);
	rec->data = malloc(rec->size);
	srec = rec->data;
	dh6db_copy(&srec->svraddr, &sc->sc_sia->sia_addr, IPV6_ADDR_LEN);

	/*
  	 *	Very dependent on structures of
	 *	struct svr_client and struct svr_record !!!
	 */
	dh6db_copy(&srec->lladdr, &sc->sc_lladdr, 
		sizeof(struct svr_record) - IPV6_ADDR_LEN);

	gettimeofday(&tv,NULL);
	srec->prefer_life = srec->prefer_life*DH6_LIFETIME_UNIT 
			    + tv.tv_sec;
	srec->valid_life = srec->valid_life*DH6_LIFETIME_UNIT
			    + tv.tv_sec;
}

int dh6db_convert_rec(DBT *rec, void *entry)
{
	struct svr_client *sc = entry;
	struct svr_record *srec = rec->data;
	struct timeval tv;

	if (rec->data == NULL || rec->size == 0) {
		log_warn(3213, "Record is NULL");
		return -1;
	}
	if ((sc->sc_sia=sia_get(&srec->svraddr)) == NULL) {
		log_warn(3214, "Server address not on list");
		return -1;
	}

	/*
  	 *	Very dependent on structures of
	 *	struct svr_client and struct svr_record !!!
	 */
	dh6db_copy(&sc->sc_lladdr, &srec->lladdr, 
		sizeof(struct svr_record) - IPV6_ADDR_LEN);

	gettimeofday(&tv,NULL);
	sc->sc_prefer_life = (sc->sc_prefer_life - tv.tv_sec) /
				DH6_LIFETIME_UNIT;
	sc->sc_valid_life = (sc->sc_valid_life - tv.tv_sec) /
				DH6_LIFETIME_UNIT;
}

int dh6db_any_idle_dh6addr( void )
{
	DB  *db2;
	DBT key, rec;

#ifdef DH6_DEBUG
        if ( look_db() )
		log_info("dh6db_any_idle_dh6addr()");
#endif
       	if ( (db2=dh6db_open(DH6_SVR_INVAL)) == NULL)
              	log_sys(3215, "dbopen error");

        memset(&key, 0, sizeof(key));
	key.size = KPREFIX_LEN;
	key.data = &key_prefix;
	db2->seq(db2, &key, &rec, R_CURSOR);

	if (rec.size == sizeof(struct svr_record)) {
#ifdef DH6_DEBUG
	        if ( look_db_more() )
			log_info("database not empty");
#endif
		dh6db_close(db2);
		return 1;
	}
	dh6db_close(db2);
	return 0;
}

int dh6db_get_idle_dh6addr( struct svr_client *sc )
{
	DB  *db2;
	DBT key, rec;

#ifdef DH6_DEBUG
        if ( look_db() )
		log_info("dh6db_get_idle_dh6addr()");
#endif
       	if ( (db2=dh6db_open(DH6_SVR_INVAL)) == NULL)
              	log_sys(3216, "dbopen error");

        memset(&key, 0, sizeof(key));
	key.size = KPREFIX_LEN;
	key.data = &key_prefix;
	db2->seq(db2, &key, &rec, R_CURSOR);

	if (rec.size == sizeof(struct svr_record)) {
		db2->del(db2, &key, R_CURSOR);
		dh6db_convert_rec(&rec, sc);
		dh6db_close(db2);
		return 1;
	}
	dh6db_close(db2);
	return 0;
}

int dh6db_get_requested_dh6addr( struct svr_client *sc )
{
	DB  *db2;

#ifdef DH6_DEBUG
        if ( look_db() )
		log_info("dh6db_get_requested_dh6addr()");
#endif
       	if ( (db2=dh6db_open(DH6_SVR_INVAL)) == NULL)
              	log_sys(3217, "dbopen error");
	if (dh6db_locate(db2, sc) == 0) {
		dh6db_del(db2, sc, R_CURSOR);
		dh6db_close(db2);
		return 1;
	}
	dh6db_close(db2);
	return 0;
}

int dh6db_release_dh6addr( struct svr_client *sc )
{
	DB  *db, *db2;
	int ret = 0;

#ifdef DH6_DEBUG
        if ( look_db() )
		log_info("dh6db_release_dh6addr()");
#endif
       	if ( (db2=dh6db_open(DH6_SVR_INVAL)) == NULL)
              	log_sys(3218, "dbopen error");
	dh6db_put(db2, sc, R_NOOVERWRITE);
	dh6db_sync(db2);
        dh6db_close(db2);

        if ( (db=dh6db_open(DH6_SVR_INUSE)) == NULL)
                log_sys(3219, "dbopen error");
	if (dh6db_locate(db, sc) == 0) {
		dh6db_del(db, sc, R_CURSOR);
		ret = 1;
	}
        dh6db_close(db);

	return ret;
}

int dh6db_load_sc(struct svr_ifaddr *sia,
	struct in6_addr *lladdr, struct in6_addr *relay)
{
	DB *db;
	DBT rec, key;
	int ret = 0;
	struct svr_client *sc;

	if ( sc_get(sia,lladdr,relay) )
		return 0;

        key.size = 2*IPV6_ADDR_LEN; 
        key.data = malloc(key.size);
        dh6db_copy(key.data, lladdr, IPV6_ADDR_LEN);
        dh6db_copy(((u_int8_t *)key.data)+IPV6_ADDR_LEN, relay, IPV6_ADDR_LEN);

        if ( (db=dh6db_open(DH6_SVR_INUSE)) == NULL)
                log_sys(3220, "dbopen error");
	ret = db->get(db, &key, &rec, 0);
	dh6db_close(db);
	free(key.data);

	if (ret == 0 && rec.size > 0) {
#ifdef DH6_DEBUG
        if ( look_db_more() )
		log_info("load a record (size %d) to memory", rec.size);
#endif
                if ( (sc=sc_create()) == NULL )
                        return -1;
		dh6db_convert_rec(&rec, sc);
                u_queue_head(&sia->sia_clilist, sc); 
		return 0;
	} else
		return -1;
}

int svr_print_record(struct svr_record *rec)
{
	log_info("DB record: ");
	log_info("  server: %s", inet_ntop(AF_INET6, &rec->svraddr, 16, NULL));
	log_info("  lladdr: %s", inet_ntop(AF_INET6, &rec->lladdr,  16, NULL));
	log_info("  agent:  %s", inet_ntop(AF_INET6, &rec->rlyaddr, 16, NULL));
	log_info("  client: %s", inet_ntop(AF_INET6, &rec->cliaddr, 16, NULL));
	log_info("  id:%d, prefix:%d, state:%d, prefer:%ld, valid:%ld",
		rec->id, rec->prfsize, rec->state, 
		rec->prefer_life, rec->valid_life);
}

int svr_dbshow_adm( ADM_MESSAGE *msg, int len )
{
	DB  *db;
	DBT key, rec;
	struct svr_record srec;

	/* at the moment just show all the records */
	log_info("Server DB %s:", DH6_SVR_INUSE);
        if ( (db=dh6db_open(DH6_SVR_INUSE)) == NULL)
                log_sys(1501, "dbopen error");

        memset(&key, 0, sizeof(key));
	key.size = KPREFIX_LEN;
	key.data = &key_prefix;
	db->seq(db, &key, &rec, R_CURSOR);

	do {
		if (rec.size < sizeof(struct svr_record)) 
			break;
		dh6db_copy(&srec, rec.data, sizeof(struct svr_record));
		svr_print_record(&srec);
	} while (db->seq(db, &key, &rec, R_NEXT) == 0);
        dh6db_close(db);

	log_info("Server DB %s:", DH6_SVR_INVAL);
        if ( (db=dh6db_open(DH6_SVR_INVAL)) == NULL)
                log_sys(1501, "dbopen error");

        memset(&key, 0, sizeof(key));
	key.size = KPREFIX_LEN;
	key.data = &key_prefix;
	db->seq(db, &key, &rec, R_CURSOR);

	do {
		if (rec.size < sizeof(struct svr_record)) 
			break;
		dh6db_copy(&srec, rec.data, sizeof(struct svr_record));
		svr_print_record(&srec);
	} while (db->seq(db, &key, &rec, R_NEXT) == 0);
        dh6db_close(db);
	return 0;
}
